from charmer import cmd, setup_handler


def run_tests(extra_args):
    out, err, exit_code = cmd.exec_python(
        "-m", "unittest", "discover", "tests", *extra_args
    )
    return exit_code


def run_lint(extra_args):
    proj_name = setup_handler.read().name
    out, _, exit_code = cmd.exec_python("-m", "pylint", proj_name, *extra_args)
    return exit_code


def build_wheel(extra_args):
    # cmd.exec_python("setup.py", "sdist")
    out, _, exit_code = cmd.exec_python("setup.py", "bdist_wheel", *extra_args)
    return exit_code


def start_project(extra_args):
    proj_name = setup_handler.read().name
    out, _, exit_code = cmd.exec_python("-m", proj_name, *extra_args)
    return exit_code



import os
import re


class Project:
    def __init__(self, name):
        self.name = name


def read(directory="."):
    setup_path = os.path.abspath(os.path.join(directory, "setup.py"))
    with open(setup_path, "r") as setup_file:
        raw_content = setup_file.read()
        raw_setup_call = _get_setup_call(raw_content)
        name = _get_key_("name", raw_setup_call)
    return Project(name)


def _get_key_(key, raw_setup_call):
    match = re.search(r"%s\s*=\s*[\'\"](.+?)[\'\"]" % key, raw_setup_call, re.DOTALL)
    return match.group(1)


def _get_setup_call(raw_content):
    match = re.search(r"setup\(.*\)", raw_content, re.DOTALL)
    return match.group(0)



import os

from charmer.constants import VENV_PATH_SUFFIX_PYTHON

BR = "\n"


def get_init():
    return ""


def get_main(name, use_conf):
    result = ""
    result += '"""This module serves as the entry point of ' + name + '."""'
    result += BR
    if use_conf:
        result += "import logging"
        result += BR
        result += "import conf"
        result += BR
    result += BR
    result += BR
    result += "def main():"
    result += BR
    result += '    """The actual entry point."""'
    result += BR
    if use_conf:
        result += "    logger = logging.getLogger('name')"
        result += BR
        result += "    log_level = logging.getLevelName(conf.get('log_level', 'DEBUG').upper())"
        result += BR
        result += "    logger.setLevel(log_level)"
        result += BR
    result += "    raise NotImplementedError()"
    result += BR
    result += BR
    result += BR
    result += "if __name__ == '__main__':"
    result += BR
    result += "    main()"
    result += BR
    return result


def get_readme(name, description, runnable, use_conf):
    result = name + BR + len(name) * "=" + BR + BR + description + BR
    if runnable:
        if use_conf:
            result += BR
            result += "Usage"
            result += BR
            result += "'''''"
            result += BR
            result += BR
            result += "::"
            result += BR
            result += BR
            result += "    python -m "
            result += name
            result += " --config config-default.yml"
            result += BR
        else:
            result += BR
            result += "Usage"
            result += BR
            result += "'''''"
            result += BR
            result += BR
            result += "::"
            result += BR
            result += BR
            result += "    python -m "
            result += name
            result += BR
    result += BR
    result += "Testing"
    result += BR
    result += "'''''''"
    result += BR
    result += BR
    result += "::"
    result += BR
    result += BR
    result += "    python -m unittest discover tests"
    result += BR
    return result


def get_setup(name, version, author, description, runnable, use_conf):
    result = '"""This is the installation toolset for this project."""'
    result += BR
    result += "from setuptools import setup, find_packages"
    result += BR
    result += BR
    result += "with open('README.rst', 'r') as fh:"
    result += BR
    result += "    long_description = fh.read()"
    result += BR
    result += BR
    result += "setup(name='"
    result += name
    result += "',"
    result += BR
    result += "      version='"
    result += version
    result += "',"
    result += BR
    result += "      author='"
    result += author
    result += "',"
    result += BR
    result += "      description='"
    result += description
    result += "',"
    result += BR
    result += "      long_description=long_description,"
    result += BR
    result += "      packages=find_packages(exclude=('tests',))"
    if use_conf:
        result += "," + BR
        result += "      install_requires=['conf~=0.2.0']"
    if runnable:
        result += ","
        result += BR
        result += "      entry_points={"
        result += BR
        result += "          'console_scripts': ["
        result += BR
        result += "              '"
        result += name
        result += " = "
        result += name
        result += ".__main__:main'"
        result += BR
        result += "          ]"
        result += BR
        result += "      }"
    result += ")" + BR
    return result


def get_context():
    result = '"""Import this module first before importing your project stuff in the tests"""'
    result += BR
    result += "import sys"
    result += BR
    result += "import os"
    result += BR
    result += BR
    result += "dname = os.path.dirname(__file__)"
    result += BR
    result += "relpath = os.path.join(dname, '..')"
    result += BR
    result += "abspath = os.path.abspath(relpath)"
    result += BR
    result += "sys.path.insert(0, abspath)"
    result += BR
    return result


def get_main_test_suite(name, runnable):
    result = '"""Contains a test suite for basic tests."""'
    result += BR
    result += "import context"
    result += BR
    result += "import unittest"
    result += BR
    if runnable:
        result += "from " + name + ".__main__ import main" + BR
    result += BR
    result += BR
    result += "class MainTestSuite(unittest.TestCase):"
    result += BR
    result += '    """Basic test cases."""'
    result += BR
    if runnable:
        result += BR
        result += "    def test_main(self):"
        result += BR
        result += "        with self.assertRaises(NotImplementedError):"
        result += BR
        result += "            main()"
        result += BR
    result += BR
    result += BR
    result += "if __name__ == '__main__':"
    result += BR
    result += "    unittest.main()"
    result += BR
    return result


def get_conf():
    result = "log_level: INFO"
    return result


def get_dev_requirements(use_pylint):
    result = ""
    result += "wheel" + BR
    result += "setuptools" + BR
    if use_pylint:
        result += "pylint" + BR
    return result



import os
from charmer.questionnaire import PreferenceInput


if os.name == "nt":
    _python_exc = "python"
    ENV_USER = "USERNAME"
else:
    _python_exc = "python3"
    ENV_USER = "USER"


PREFERENCE_INPUTS = [
    PreferenceInput(
        "name",
        "Project name",
        default_value=os.getcwd().split(os.sep)[-1].replace("-", "_"),
        regex="[a-zA-Z_]\w*",
        requirements="A project name must be a word; it must start with a "
        "letter or underscore and may contain only letters, "
        "underscores and numbers in the remainder.",
    ),
    PreferenceInput(
        "description", "Description", default_value="My awesome Python project"
    ),
    PreferenceInput(
        "version",
        "Initial version",
        default_value="0.1.0",
        regex="[0-9]\.[0-9]\.[0-9]",
        requirements="A version must have the form: x.y.z where x, y and z are all whole numbers.",
    ),
    PreferenceInput(
        "author", "Author", default_value=os.getenv(ENV_USER), regex="([a-zA-Z_]\w*)?"
    ),
    PreferenceInput("dir", "Output directory", default_value="./"),
    PreferenceInput(
        "runnable", "Is the application runnable?", default_value="yes", yes_or_no=True
    ),
    PreferenceInput(
        "create venv",
        "Create virtual environment?",
        default_value="yes",
        yes_or_no=True,
    ),
    PreferenceInput("use conf", "Use conf?", default_value="yes", yes_or_no=True),
    PreferenceInput("use pylint", "Use Pylint?", default_value="yes", yes_or_no=True),
]



import os
import sys
from charmer import cmd, questionnaire
from charmer.constants import VENV_NAME
from charmer.inputs import PREFERENCE_INPUTS
from charmer.templates import (
    get_main,
    get_init,
    get_readme,
    get_setup,
    get_context,
    get_main_test_suite,
    get_conf,
    get_dev_requirements,
)


def start(args):
    preferences_by_name = questionnaire.get_preferences(PREFERENCE_INPUTS, args)
    verbose = args.verbose
    name = preferences_by_name["name"].value
    output_dir = preferences_by_name["dir"].value
    description = preferences_by_name["description"].value
    version = preferences_by_name["version"].value
    author = preferences_by_name["author"].value
    runnable = preferences_by_name["runnable"].value
    create_venv = preferences_by_name["create venv"].value
    use_conf = preferences_by_name["use conf"].value
    use_pylint = preferences_by_name["use pylint"].value

    if os.listdir(output_dir):
        print(
            "Could not initialize project. The output directory ('%s') is not empty."
            % output_dir
        )
        return 1

    start_str = "Initializing project '%s'" % name
    print("")
    print(start_str)
    print(len(start_str) * "-")

    main_package = os.path.join(output_dir, name)
    tests_package = os.path.join(output_dir, "tests")

    sys.stdout.write("> Creating main package... ")
    os.makedirs(main_package, exist_ok=True)
    _create_file("__init__.py", get_init(), main_package)
    print("done")

    sys.stdout.write("> Creating test package... ")
    os.makedirs(tests_package, exist_ok=True)
    _create_file("__init__.py", get_init(), tests_package)
    _create_file("context.py", get_context(), tests_package)
    _create_file("test_main.py", get_main_test_suite(name, runnable), tests_package)
    print("done")

    sys.stdout.write("> Creating README.rst... ")
    _create_file(
        "README.rst", get_readme(name, description, runnable, use_conf), output_dir
    )
    print("done")

    sys.stdout.write("> Creating setup.py... ")
    _create_file(
        "setup.py",
        get_setup(name, version, author, description, runnable, use_conf),
        output_dir,
    )
    print("done")

    sys.stdout.write("> Creating requirements.txt... ")
    _create_file("requirements.txt", get_dev_requirements(use_pylint), output_dir)
    print("done")

    sys.stdout.write("> Creating entrypoint... ")
    _create_file("__main__.py", get_main(name, use_conf), main_package)
    print("done")

    if use_conf:
        sys.stdout.write("> Creating default configuration... ")
        _create_file("config-default.yml", get_conf(), output_dir)
        print("done")

    if create_venv:
        sys.stdout.write("> Creating virtual environment... ")
        sys.stdout.flush()
        cmd.exec_python(
            "-m", "venv", os.path.join(output_dir, VENV_NAME), print_output=verbose
        )
        cmd.exec_python(
            "-m", "pip", "install", "--upgrade", "pip", print_output=verbose
        )
        print("done")

    sys.stdout.write("> Installing %s... " % name)
    sys.stdout.flush()
    cmd.exec_pip("install", "-e", ".", print_output=verbose)
    print("done")

    sys.stdout.write("> Installing dev dependencies... ")
    sys.stdout.flush()
    cmd.exec_pip("install", "-r", "requirements.txt", print_output=verbose)
    print("done")

    print("")
    print("Finished!")
    return 0  # TODO beter dit


def _create_file(fname, content, pname=None):
    output_path = os.path.abspath(os.path.join(pname, fname))
    with open(output_path, "w+") as f:
        f.write(content)



import argparse
import sys
from collections import OrderedDict
from charmer import initializer, runner


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "action", help="Initialize a new project", nargs="*", default="init"
    )
    parser.add_argument(
        "-y",
        "--yes",
        action="store_true",
        help="Say 'yes' to all prompts during initialization",
    )
    parser.add_argument(
        "-vb",
        "--verbose",
        action="store_true",
        help="Show output of sub processes during initialization",
    )
    args = parser.parse_known_args()[0]
    actions = OrderedDict(
        [
            ("test", runner.run_tests),
            ("lint", runner.run_lint),
            ("start", runner.start_project),
            ("wheel", runner.build_wheel),
        ]
    )
    exit_code = 0

    args.action = sys.argv or "init"

    if "init" in args.action:
        initializer.start(args)
    for action_name in actions:
        if action_name in args.action:
            extra_args = sys.argv[sys.argv.index(action_name) + 1 :]
            if "_" in extra_args:
                extra_args = extra_args[: extra_args.index("_")]
            exit_code |= actions[action_name](extra_args)
    exit(exit_code)


if __name__ == "__main__":
    main()

# TODO add command: install --dev
# TODO add command: clean
# TODO add command: upload



import collections
import re


try:
    _input = raw_input  # Python2.7
except NameError:
    _input = input  # Python3.+


Preference = collections.namedtuple("Preference", ["name", "value"])


def get_preferences(preference_inputs, cmd_args):
    if cmd_args.yes:
        return {
            pref_input._name: pref_input.default() for pref_input in preference_inputs
        }

    preferences = [pref_input.get() for pref_input in preference_inputs]
    _print_overview(preferences)
    correct = _input("Is this correct? (yes) ").lower().strip() or "yes"
    if correct not in ("yes", "y"):
        print("")
        return get_preferences(preference_inputs, cmd_args)
    return {pref.name: pref for pref in preferences}


class PreferenceInput:
    def __init__(
        self,
        name,
        description,
        default_value,
        *,
        yes_or_no=False,
        regex=None,
        requirements=None
    ):
        self._name = name
        self._description = description
        self._default_value = default_value
        self._yes_or_no = yes_or_no
        self._regex = regex
        self._requirements = requirements

    def get(self):
        descr = self._description
        deflt = self._default_value
        value = _input("%s (%s) " % (descr, deflt)) or deflt
        if self._regex:
            match = re.match(self._regex, value)
            if not match or match.group(0) != value:
                if self._requirements:
                    print("Invalid input. %s" % self._requirements)
                else:
                    print("Invalid input.")
                print("")
                return self.get()
        if self._yes_or_no:
            value = value.lower() in ("yes", "y")
        return Preference(self._name, value)

    def default(self):
        return Preference(self._name, self._default_value)


def _print_overview(preferences):
    print("")
    print("Setup overview")
    print("--------------")
    sorted_by_name = list(preferences)
    sorted_by_name.sort(key=lambda pref: len(pref.name))
    longest_name_len = len(sorted_by_name[-1].name)
    for pref in preferences:
        len_dif = longest_name_len - len(pref.name)
        print("%s: %s%s" % (pref.name, (len_dif * " "), pref.value))
    print("")



import os


if os.name == "nt":
    PYTHON_EXC = "python"
    PIP_EXC = "pip"
    ENV_USER = "USERNAME"
    VENV_PATH_SUFFIX_PIP = "Scripts/pip"
    VENV_PATH_SUFFIX_PYTHON = "Scripts/python"
    OS_IS_WINDOWS = True
else:
    PYTHON_EXC = "python3"
    PIP_EXC = "pip3"
    ENV_USER = "USER"
    VENV_PATH_SUFFIX_PIP = "bin/pip3"
    VENV_PATH_SUFFIX_PYTHON = "bin/python3"
    OS_IS_WINDOWS = False
VENV_NAME = "virtual_env"



"""This is the installation toolset for this project."""
from setuptools import setup, find_packages

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name="charmer",
    version="0.1.0",
    author="Ramon Hagenaars",
    description="A commandline interface for creating project structures",
    long_description=long_description,
    packages=find_packages(exclude=("tests",)),
    entry_points={"console_scripts": ["charm=charmer.__main__:main"]},
)



import os
from subprocess import Popen, PIPE, check_output
from charmer.constants import (
    OS_IS_WINDOWS,
    VENV_NAME,
    VENV_PATH_SUFFIX_PIP,
    VENV_PATH_SUFFIX_PYTHON,
    PIP_EXC,
    PYTHON_EXC,
)


def exec_python(*args, working_dir=".", print_output=True):
    python_cmd = PYTHON_EXC
    if VENV_NAME in os.listdir(working_dir):
        python_cmd = os.path.abspath(os.path.join(VENV_NAME, VENV_PATH_SUFFIX_PYTHON))
    return exec(python_cmd, *args, print_output=print_output)


def exec_pip(*args, working_dir=".", print_output=True):
    pip_cmd = PIP_EXC
    if VENV_NAME in os.listdir(working_dir):
        pip_cmd = os.path.abspath(os.path.join(VENV_NAME, VENV_PATH_SUFFIX_PIP))
    return exec(pip_cmd, *args, print_output=print_output)


def exec(*commands, print_output=True):
    with Popen(commands, shell=OS_IS_WINDOWS, stdout=PIPE, stderr=PIPE) as proc:
        outb, errb = proc.communicate()
        out = outb.decode("utf8")
        err = errb.decode("utf8")
        if print_output:
            if out:
                print(out)
            if err:
                print(err)
        if proc.returncode:
            msg = "Command failed: %s" % " ".join(commands)
            if out:
                msg += "\n" + out
            if err:
                msg += "\n" + err
            raise Exception(msg)
        return out, err, proc.returncode
